@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2003, 2004, 2005
+ * Copyright (c) 2003, 2004, 2005, 2006
  * 	Lev Walkin <vlm@lionet.info>. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: asn1c.c,v 1.26 2005/06/05 09:42:42 vlm Exp $
+ * $Id: asn1c.c,v 1.40 2006/09/12 05:22:12 vlm Exp $
  */
 /*
  * This is the program that connects the libasn1* libraries together.
@@ -31,6 +31,10 @@
  */
 #include "sys-common.h"
 
+#undef  COPYRIGHT
+#define COPYRIGHT       \
+	"Copyright (c) 2003, 2004, 2005, 2006 Lev Walkin <vlm@lionet.info>\n"
+
 #include <asn1parser.h>		/* Parse the ASN.1 file and build a tree */
 #include <asn1fix.h>		/* Fix the ASN.1 tree */
 #include <asn1print.h>		/* Print the ASN.1 tree */
@@ -38,11 +42,15 @@
 
 #include <asn1c_compat.h>	/* Portable basename(3) and dirname(3) */
 
-#undef  COPYRIGHT
-#define COPYRIGHT       \
-	"Copyright (c) 2003, 2004, 2005 Lev Walkin <vlm@lionet.info>\n"
+#ifdef	WIN32
+#include <io.h>
+#include <direct.h>
+#else
+#include <dirent.h>
+#endif
 
 static void usage(const char *av0);	/* Print the Usage screen and exit */
+static int importStandardModules(asn1p_t *asn, const char *skeletons_dir);
 
 int
 main(int ac, char **av) {
@@ -62,7 +70,7 @@ main(int ac, char **av) {
 	/*
 	 * Process command-line options.
 	 */
-	while((ch = getopt(ac, av, "EFf:hLPp:RS:vW:X")) != -1)
+	while((ch = getopt(ac, av, "EFf:g:hLPp:RS:vW:X")) != -1)
 	switch(ch) {
 	case 'E':
 		print_arg__print_out = 1;
@@ -77,13 +85,13 @@ main(int ac, char **av) {
 			asn1_fixer_flags |= A1F_EXTENDED_SizeConstraint;
 		} else if(strcmp(optarg, "compound-names") == 0) {
 			asn1_compiler_flags |= A1C_COMPOUND_NAMES;
+		} else if(strcmp(optarg, "indirect-choice") == 0) {
+			asn1_compiler_flags |= A1C_INDIRECT_CHOICE;
 		} else if(strncmp(optarg, "known-extern-type=", 18) == 0) {
 			char *known_type = optarg + 18;
 			ret = asn1f_make_known_external_type(known_type);
 			assert(ret == 0 || errno == EEXIST);
-		} else if(
-			strcmp(optarg, "native-integers") == 0 /* Legacy */
-			|| strcmp(optarg, "native-types") == 0) {
+		} else if(strcmp(optarg, "native-types") == 0) {
 			asn1_compiler_flags |= A1C_USE_NATIVE_TYPES;
 		} else if(strcmp(optarg, "no-constraints") == 0) {
 			asn1_compiler_flags |= A1C_NO_CONSTRAINTS;
@@ -91,13 +99,21 @@ main(int ac, char **av) {
 			asn1_compiler_flags |= A1C_NO_INCLUDE_DEPS;
 		} else if(strcmp(optarg, "unnamed-unions") == 0) {
 			asn1_compiler_flags |= A1C_UNNAMED_UNIONS;
-		} else if(strcmp(optarg, "types88") == 0) {
-			asn1_parser_flags |= A1P_TYPES_RESTRICT_TO_1988;
+		} else if(strcmp(optarg, "skeletons-copy") == 0) {
+			asn1_compiler_flags |= A1C_SKELETONS_COPY;
 		} else {
 			fprintf(stderr, "-f%s: Invalid argument\n", optarg);
 			exit(EX_USAGE);
 		}
 		break;
+	case 'g':
+		if(strcmp(optarg, "en-PER") == 0) {
+			asn1_compiler_flags |= A1C_GEN_PER;
+		} else {
+			fprintf(stderr, "-g%s: Invalid argument\n", optarg);
+			exit(EX_USAGE);
+		}
+		break;
 	case 'h':
 		usage(av[0]);
 	case 'P':
@@ -105,8 +121,18 @@ main(int ac, char **av) {
 		asn1_compiler_flags &= ~A1C_NO_C99;
 		break;
 	case 'p':
-		if(strcmp(optarg, "rint-constraints") == 0) {
-			asn1_printer_flags |= APF_DEBUG_CONSTRAINTS;
+		if(strncmp(optarg, "du=", 3) == 0) {
+			char *pduname = optarg + 3;
+			if(strcmp(pduname, "auto")) {
+				fprintf(stderr, "-pdu=%s: expected -pdu=auto\n",
+					pduname);
+				exit(EX_USAGE);
+			}
+			asn1_compiler_flags |= A1C_PDU_AUTO;
+		} else if(strcmp(optarg, "rint-class-matrix") == 0) {
+			asn1_printer_flags |= APF_PRINT_CLASS_MATRIX;
+		} else if(strcmp(optarg, "rint-constraints") == 0) {
+			asn1_printer_flags |= APF_PRINT_CONSTRAINTS;
 		} else if(strcmp(optarg, "rint-lines") == 0) {
 			asn1_printer_flags |= APF_LINE_COMMENTS;
 		} else {
@@ -173,11 +199,46 @@ main(int ac, char **av) {
 		ac -= optind;
 		av += optind;
 	} else {
-		fprintf(stderr, "%s: No input files specified\n",
-			a1c_basename(av[0]));
+		char *bin_name = a1c_basename(av[0]);
+		fprintf(stderr, "%s: No input files specified. "
+			"Try '%s -h' for more information\n",
+			bin_name, bin_name);
 		exit(1);
 	}
 
+	/*
+	 * Make sure the skeleton directory is out there.
+	 */
+	if(skeletons_dir == NULL) {
+		struct stat sb;
+		skeletons_dir = DATADIR;
+		if((av[-optind][0] == '.' || av[-optind][1] == '/')
+		&& stat(skeletons_dir, &sb)) {
+			/*
+			 * The default skeletons directory does not exist,
+			 * compute it from my file name:
+			 * ./asn1c/asn1c -> ./skeletons
+			 */
+			char *p;
+			size_t len;
+
+			p = a1c_dirname(av[-optind]);
+
+			len = strlen(p) + sizeof("/../skeletons");
+			skeletons_dir = malloc(len);
+			assert(skeletons_dir);
+			snprintf(skeletons_dir, len, "%s/../skeletons", p);
+			if(stat(skeletons_dir, &sb)) {
+				fprintf(stderr,
+					"WARNING: skeletons are neither in "
+					"\"%s\" nor in \"%s\"!\n",
+					DATADIR, skeletons_dir);
+				if(warnings_as_errors)
+					exit(EX_OSFILE);
+			}
+		}
+	}
+
 	/*
 	 * Iterate over input files and parse each.
 	 * All syntax trees from all files will be bundled together.
@@ -198,13 +259,16 @@ main(int ac, char **av) {
 			asn1p_module_t *mod;
 			while((mod = TQ_REMOVE(&(new_asn->modules), mod_next)))
 				TQ_ADD(&(asn->modules), mod, mod_next);
-			asn1p_free(new_asn);
+			asn1p_delete(new_asn);
 		} else {
 			asn = new_asn;
 		}
-
 	}
 
+	/* These are mostly notes for the human readers */
+	assert(asn);
+	assert(skeletons_dir);
+
 	/*
 	 * Dump the parsed ASN.1 tree if -E specified and -F is NOT given.
 	 */
@@ -214,6 +278,13 @@ main(int ac, char **av) {
 		return 0;
 	}
 
+	/*
+	 * Read in the files from skeletons/standard-modules
+	 */
+	if(importStandardModules(asn, skeletons_dir)) {
+		if(warnings_as_errors)
+			exit(EX_DATAERR);
+	}
 
 	/*
 	 * Process the ASN.1 specification: perform semantic checks,
@@ -241,50 +312,113 @@ main(int ac, char **av) {
 		return 0;
 	}
 
-	/*
-	 * Make sure the skeleton directory is out there.
-	 */
-	if(skeletons_dir == NULL) {
-		struct stat sb;
-		skeletons_dir = DATADIR;
-		if((av[-optind][0] == '.' || av[-optind][1] == '/')
-		&& stat(skeletons_dir, &sb)) {
-			/*
-			 * The default skeletons directory does not exist,
-			 * compute it from my file name:
-			 * ./asn1c/asn1c -> ./skeletons
-			 */
-			char *p;
-			size_t len;
-
-			p = a1c_dirname(av[-optind]);
-
-			len = strlen(p) + sizeof("/../skeletons");
-			skeletons_dir = alloca(len);
-			snprintf(skeletons_dir, len, "%s/../skeletons", p);
-			if(stat(skeletons_dir, &sb)) {
-				fprintf(stderr,
-					"WARNING: skeletons are neither in "
-					"\"%s\" nor in \"%s\"!\n",
-					DATADIR, skeletons_dir);
-				if(warnings_as_errors)
-					exit(EX_SOFTWARE);
-			}
-		}
-	}
-
 	/*
 	 * Compile the ASN.1 tree into a set of source files
 	 * of another language.
 	 */
 	if(asn1_compile(asn, skeletons_dir, asn1_compiler_flags,
-			ac + optind, av - optind)) {
+			ac + optind, optind - 1, av - optind)) {
 		exit(EX_SOFTWARE);
 	}
 
 	return 0;
 }
 
+/*
+ * Parse and import *.asn1 from skeletons/standard-modules
+ */
+static int
+importStandardModules(asn1p_t *asn, const char *skeletons_dir) {
+	asn1p_t *new_asn;
+	asn1p_module_t *mod;
+	const char *filename;
+	char *fullname;
+	char *target_dir;
+	int target_dir_len;
+	int len;
+#ifdef	WIN32
+	intptr_t dir;
+	struct _finddata_t c_file;
+	char *pattern;
+#else
+	struct dirent *dp;
+	DIR *dir;
+#endif
+	int ret = 0;
+
+	/* Notes for the human reader */
+	assert(asn);
+	assert(skeletons_dir);
+
+	/*
+	 * Figure out the standard-modules directory.
+	 */
+	target_dir_len = strlen(skeletons_dir)
+				+ sizeof("/standard-modules") - 1;
+	target_dir = malloc(target_dir_len + 1);
+	assert(target_dir);
+	snprintf(target_dir, target_dir_len + 1, "%s/standard-modules",
+		skeletons_dir);
+
+#ifdef	WIN32
+	len = target_dir_len + sizeof("/*.asn1");
+	pattern = malloc(len);
+	assert(pattern);
+	snprintf(pattern, len, "%s/*.asn1", target_dir);
+	dir = _findfirst(pattern, &c_file);
+	if(dir == -1L) {
+#else
+	dir = opendir(target_dir);
+	if(!dir) {
+#endif
+		fprintf(stderr,
+			"WARNING: Cannot find standard modules in %s\n",
+			target_dir);
+		return -1;
+	}
+
+#ifdef	WIN32
+	do {
+		filename = c_file.name;
+#else
+	while((dp = readdir(dir))) {
+		filename = dp->d_name;
+#endif
+		len = strlen(filename);
+		if(len <= 5 || strcmp(filename + len - 5, ".asn1"))
+			continue;
+		len = target_dir_len + 1 + len + 1;
+		fullname = malloc(len);
+		if(!fullname) continue;	/* Just skip it, no big deal */
+		snprintf(fullname, len, "%s/%s", target_dir, filename);
+		filename = fullname;
+
+		new_asn = asn1p_parse_file(filename, A1P_NOFLAGS);
+		if(new_asn == NULL) {
+			fprintf(stderr, "WARNING: Cannot parse standard module \"%s\"\n", filename);
+			ret = -1;
+			continue;
+		}
+
+		/* Import these modules and mark them as "standard" */
+		while((mod = TQ_REMOVE(&(new_asn->modules), mod_next))) {
+			mod->_tags |= MT_STANDARD_MODULE;
+			TQ_ADD(&(asn->modules), mod, mod_next);
+		}
+		asn1p_delete(new_asn);
+
+#ifdef	WIN32
+	} while(_findnext(dir, &c_file) == 0);
+	_findclose(dir);
+#else
+		free(fullname);
+	} /* while(readdir()) */
+	closedir(dir);
+#endif
+
+	return ret;
+}
+
 /*
  * Print the usage screen and exit(EX_USAGE).
  */
@@ -310,17 +444,22 @@ usage(const char *av0) {
 "  -Wdebug-compiler      --//-- compiler\n"
 "\n"
 
-"  -fall-defs-global     Don't make the asn1_DEF_'s of structure members \"static\"\n"
 "  -fbless-SIZE          Allow SIZE() constraint for INTEGER etc (non-std.)\n"
 "  -fcompound-names      Disambiguate C's struct NAME's inside top-level types\n"
+"  -findirect-choice     Compile members of CHOICE as indirect pointers\n"
 "  -fknown-extern-type=<name>    Pretend the specified type is known\n"
 "  -fnative-types        Use \"long\" instead of INTEGER_t whenever possible, etc.\n"
 "  -fno-constraints      Do not generate constraint checking code\n"
 "  -fno-include-deps     Do not generate courtesy #includes for dependencies\n"
 "  -funnamed-unions      Enable unnamed unions in structures\n"
-"  -ftypes88             Pretend to support only ASN.1:1988 embedded types\n"
+"  -fskeletons-copy      Force copying the support files\n"
 "\n"
 
+"  -gen-PER              Generate PER support code\n"
+"  -pdu=auto             Generate PDU table (discover PDUs automatically)\n"
+"\n"
+
+"  -print-class-matrix   Print out the collected object class matrix (debug)\n"
 "  -print-constraints    Explain subtype constraints (debug)\n"
 "  -print-lines          Generate \"-- #line\" comments in -E output\n"
 
