@@ -22,7 +22,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: enber.c,v 1.11 2005/06/08 00:06:59 vlm Exp $
+ * $Id: enber.c,v 1.12 2006/09/13 02:59:46 vlm Exp $
  */
 #include "sys-common.h"
 
@@ -136,7 +136,7 @@ process(const char *fname) {
 			if((collector_size - collector_offset) <= len
 			|| !collector) {
 				collector_size <<= 1;
-				collector = realloc(collector, collector_size);
+				collector = REALLOC(collector, collector_size);
 				if(!collector) {
 					perror("realloc()");
 					exit(EX_OSERR);
