@@ -1,5 +1,6 @@
 /*-
- * Copyright (c) 2004, 2005 Lev Walkin <vlm@lionet.info>. All rights reserved.
+ * Copyright (c) 2004, 2005, 2006 Lev Walkin <vlm@lionet.info>.
+ * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -22,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: unber.c,v 1.18 2005/06/15 18:41:50 vlm Exp $
+ * $Id: unber.c,v 1.26 2006/09/13 02:59:46 vlm Exp $
  */
 #include "sys-common.h"
 
@@ -45,8 +46,10 @@ static int process(const char *fname);	/* Perform the BER decoding */
 static int decode_tlv_from_string(const char *datastring);
 
 static int single_type_decoding = 0;	/* -1 enables that */
+static int minimalistic = 0;		/* -m enables that */
 static int pretty_printing = 1;		/* -p disables that */
-static char *indent_buffer = "    ";	/* -i controls that */
+static int skip_bytes = 0;		/* -s controls that */
+static char indent_bytes[16] = "    ";	/* -i controls that */
 
 int
 main(int ac, char **av) {
@@ -56,29 +59,39 @@ main(int ac, char **av) {
 	/*
 	 * Process command-line options.
 	 */
-	while((ch = getopt(ac, av, "1hi:pt:v")) != -1)
+	while((ch = getopt(ac, av, "1hi:mps:t:v")) != -1)
 	switch(ch) {
 	case '1':
 		single_type_decoding = 1;
 		break;
-	case 't':
-		if(decode_tlv_from_string(optarg))
-			exit(EX_DATAERR);
-		exit(0);
-	case 'p':
-		pretty_printing = 0;
-		break;
 	case 'i':
 		i = atoi(optarg);
-		if(i >= 0 && i < 16) {
-			indent_buffer = alloca(i + 1);
-			memset(indent_buffer, ' ', i);
-			indent_buffer[i] = '\0';
+		if(i >= 0 && i < (int)sizeof(indent_bytes)) {
+			memset(indent_bytes, ' ', i);
+			indent_bytes[i] = '\0';
 		} else {
 			fprintf(stderr, "-i %s: Invalid indent value\n",optarg);
 			exit(EX_USAGE);
 		}
 		break;
+	case 'm':
+		minimalistic = 1;
+		break;
+	case 'p':
+		pretty_printing = 0;
+		break;
+	case 's':
+		skip_bytes = atoi(optarg);
+		if(skip_bytes < 0) {
+			fprintf(stderr, "-s %s: positive value expected\n",
+				optarg);
+			exit(EX_USAGE);
+		}
+		break;
+	case 't':
+		if(decode_tlv_from_string(optarg))
+			exit(EX_DATAERR);
+		exit(0);
 	case 'v':
 		fprintf(stderr, "ASN.1 BER Decoder, v" VERSION "\n" COPYRIGHT);
 		exit(0);
@@ -122,10 +135,12 @@ usage(const char *av0) {
 "ASN.1 BER Decoder, v" VERSION "\n" COPYRIGHT
 "Usage: %s [options] [-] [file ...]\n"
 "Options:\n"
-"  -1                 Decode only the first BER structure (otherwise, until EOF)\n"
-"  -i <indent>        Amount of spaces for output indentation (default is 4)\n"
-"  -p                 Do not attempt pretty-printing of known ASN.1 types\n"
-"  -t <data-string>   Decode the given tag[/length] sequence (e.g. -t \"bf20\")\n"
+"  -1                Decode only the first BER structure (otherwise, until EOF)\n"
+"  -i <indent>       Amount of spaces for output indentation (default is 4)\n"
+"  -m                Minimalistic mode: print as little as possible\n"
+"  -p                Do not attempt pretty-printing of known ASN.1 types\n"
+"  -s <skip>         Ignore first <skip> bytes of input\n"
+"  -t <hex-string>   Decode the given tag[/length] sequence (e.g. -t \"bf20\")\n"
 "\n"
 "The XML opening tag format is as follows:\n"
 "  <tform O=\"off\" T=\"tag\" TL=\"tl_len\" V=\"{Indefinite|v_len}\" [A=\"type\"] [F]>\n"
@@ -147,8 +162,8 @@ typedef enum pd_code {
 	PD_FINISHED	= 0,
 	PD_EOF		= 1,
 } pd_code_e;
-static pd_code_e process_deeper(const char *fname, FILE *fp, asn1c_integer_t *offset, int level, ssize_t limit, ssize_t *frame_size, int expect_eoc);
-static void print_TL(int fin, asn1c_integer_t offset, int level, int constr, ssize_t tlen, ber_tlv_tag_t, ber_tlv_len_t);
+static pd_code_e process_deeper(const char *fname, FILE *fp, asn1c_integer_t *offset, int level, ssize_t limit, ber_tlv_len_t *frame_size, ber_tlv_len_t effective_size, int expect_eoc);
+static void print_TL(int fin, asn1c_integer_t offset, int level, int constr, ssize_t tlen, ber_tlv_tag_t, ber_tlv_len_t, ber_tlv_len_t effective_frame_size);
 static int print_V(const char *fname, FILE *fp, ber_tlv_tag_t, ber_tlv_len_t);
 
 /*
@@ -158,11 +173,11 @@ static int
 process(const char *fname) {
 	FILE *fp;
 	pd_code_e pdc;
-	asn1c_integer_t offset = 0;	/* Stream decoding position */
-	ssize_t frame_size = 0;		/* Single frame size */
+	asn1c_integer_t offset = 0;		/* Stream decoding position */
+	ber_tlv_len_t frame_size = 0;		/* Single frame size */
 
 	if(strcmp(fname, "-")) {
-		fp = fopen(fname, "r");
+		fp = fopen(fname, "rb");
 		if(!fp) {
 			perror(fname);
 			return -1;
@@ -171,11 +186,27 @@ process(const char *fname) {
 		fp = stdin;
 	}
 
+	/*
+	 * Skip the requested amount of bytes.
+	 */
+	for(; offset < skip_bytes; offset++) {
+		if(fgetc(fp) == -1) {
+			fprintf(stderr,
+				"%s: input source (%" PRIdASN " bytes) "
+				"has less data than \"-s %d\" switch "
+				"wants to skip\n",
+				fname, offset, skip_bytes);
+			if(fp != stdin) fclose(fp);
+			return -1;
+		}
+	}
+
 	/*
 	 * Fetch out BER-encoded data until EOF or error.
 	 */
 	do {
-		pdc = process_deeper(fname, fp, &offset, 0, -1, &frame_size, 0);
+		pdc = process_deeper(fname, fp, &offset,
+				     0, -1, &frame_size, 0, 0);
 	} while(pdc == PD_FINISHED && !single_type_decoding);
 
 	if(fp != stdin)
@@ -189,7 +220,8 @@ process(const char *fname) {
 /*
  * Process the TLV recursively.
  */
-static pd_code_e process_deeper(const char *fname, FILE *fp, asn1c_integer_t *offset, int level, ssize_t limit, ssize_t *frame_size, int expect_eoc) {
+static pd_code_e
+process_deeper(const char *fname, FILE *fp, asn1c_integer_t *offset, int level, ssize_t limit, ber_tlv_len_t *frame_size, ber_tlv_len_t effective_size, int expect_eoc) {
 	unsigned char tagbuf[32];
 	ssize_t tblen = 0;
 	pd_code_e pdc = PD_FINISHED;
@@ -198,7 +230,8 @@ static pd_code_e process_deeper(const char *fname, FILE *fp, asn1c_integer_t *of
 	ssize_t t_len;
 	ssize_t l_len;
 
-	do {
+	for(;;) {
+		ber_tlv_len_t local_esize = 0;
 		int constr;
 		int ch;
 
@@ -217,7 +250,7 @@ static pd_code_e process_deeper(const char *fname, FILE *fp, asn1c_integer_t *of
 		/* Get the next byte from the input stream */
 		ch = fgetc(fp);
 		if(ch == -1) {
-			if(tblen || limit > 0) {
+			if(limit > 0 || expect_eoc) {
 				fprintf(stderr,
 					"%s: Unexpected end of file (TL)"
 					" at %" PRIdASN "\n",
@@ -267,7 +300,8 @@ static pd_code_e process_deeper(const char *fname, FILE *fp, asn1c_integer_t *of
 		assert((t_len + l_len) == tblen);
 
 		if(!expect_eoc || tagbuf[0] || tagbuf[1])
-			print_TL(0, *offset, level, constr, tblen, tlv_tag, tlv_len);
+			print_TL(0, *offset, level, constr, tblen,
+				 tlv_tag, tlv_len, effective_size);
 
 		if(limit != -1) {
 			/* If limit is set, account for the TL sequence */
@@ -285,15 +319,18 @@ static pd_code_e process_deeper(const char *fname, FILE *fp, asn1c_integer_t *of
 
 		*offset += t_len + l_len;
 		*frame_size += t_len + l_len;
+		effective_size += t_len + l_len;
+		local_esize += t_len + l_len;
 
-		if(expect_eoc && tagbuf[0] == '\0' && tagbuf[1] == '\0') {
+		if(expect_eoc && !tagbuf[0] && !tagbuf[1]) {
 			/* End of content octets */
-			print_TL(1, *offset, level - 1, 1, 2, 0, -1);
+			print_TL(1, *offset - 2, level - 1, 1, 2, 0, -1,
+					effective_size);
 			return PD_FINISHED;
 		}
 
 		if(constr) {
-			ssize_t dec = 0;
+			ber_tlv_len_t dec = 0;
 			/*
 			 * This is a constructed type. Process recursively.
 			 */
@@ -303,15 +340,20 @@ static pd_code_e process_deeper(const char *fname, FILE *fp, asn1c_integer_t *of
 			}
 			pdc = process_deeper(fname, fp, offset, level + 1,
 				tlv_len == -1 ? limit : tlv_len,
-				&dec, tlv_len == -1);
+				&dec, t_len + l_len, tlv_len == -1);
 			if(pdc == PD_FAILED) return pdc;
 			if(limit != -1) {
 				assert(limit >= dec);
 				limit -= dec;
 			}
 			*frame_size += dec;
+			effective_size += dec;
+			local_esize += dec;
 			if(tlv_len == -1) {
 				tblen = 0;
+				if(pdc == PD_FINISHED
+					&& limit < 0 && !expect_eoc)
+					return pdc;
 				continue;
 			}
 		} else {
@@ -325,38 +367,45 @@ static pd_code_e process_deeper(const char *fname, FILE *fp, asn1c_integer_t *of
 			}
 			*offset += tlv_len;
 			*frame_size += tlv_len;
+			effective_size += tlv_len;
+			local_esize += tlv_len;
 		}
 
-		print_TL(1, *offset, level, constr, tblen, tlv_tag, tlv_len);
+		print_TL(1, *offset, level, constr, tblen,
+			 tlv_tag, tlv_len, local_esize);
 
 		tblen = 0;
-	} while(1);
+
+		/* Report success for a single top level TLV */
+		if(level == 0 && limit == -1 && !expect_eoc)
+			break;
+	} /* for(;;) */
 
 	return pdc;
 }
 
 static void
-print_TL(int fin, asn1c_integer_t offset, int level, int constr, ssize_t tlen, ber_tlv_tag_t tlv_tag, ber_tlv_len_t tlv_len) {
+print_TL(int fin, asn1c_integer_t offset, int level, int constr, ssize_t tlen, ber_tlv_tag_t tlv_tag, ber_tlv_len_t tlv_len, ber_tlv_len_t effective_size) {
 
 	if(fin && !constr) {
 		printf("</P>\n");
 		return;
 	}
 
-	while(level-- > 0) printf(indent_buffer);  /* Print indent */
+	while(level-- > 0) printf(indent_bytes);  /* Print indent */
 	printf(fin ? "</" : "<");
 
 	printf(constr ? ((tlv_len == -1) ? "I" : "C") : "P");
 
-	/* In case of <P>, <C>, <I>, </I> print out the offset */
-	if(fin == 0 || tlv_len == -1)
+	/* Print out the offset of this boundary, even if closing tag */
+	if(!minimalistic)
 		printf(" O=\"%" PRIdASN "\"", offset);
 
 	printf(" T=\"");
 	ber_tlv_tag_fwrite(tlv_tag, stdout);
 	printf("\"");
 
-	if(!fin || tlv_len == -1)
+	if(!fin || (tlv_len == -1 && !minimalistic))
 		printf(" TL=\"%ld\"", (long)tlen);
 	if(!fin) {
 		if(tlv_len == -1)
@@ -365,14 +414,19 @@ print_TL(int fin, asn1c_integer_t offset, int level, int constr, ssize_t tlen, b
 			printf(" V=\"%ld\"", (long)tlv_len);
 	}
 
-	if(BER_TAG_CLASS(tlv_tag) == ASN_TAG_CLASS_UNIVERSAL) {
+	if(!minimalistic
+	&& BER_TAG_CLASS(tlv_tag) == ASN_TAG_CLASS_UNIVERSAL) {
 		const char *str;
 		ber_tlv_tag_t tvalue = BER_TAG_VALUE(tlv_tag);
 		str = ASN_UNIVERSAL_TAG2STR(tvalue);
 		if(str) printf(" A=\"%s\"", str);
 	}
 
-	if(fin) printf(">\n");
+	if(fin) {
+		if(constr && !minimalistic)
+			printf(" L=\"%ld\"", (long)effective_size);
+		printf(">\n");
+	}
 }
 
 /*
@@ -418,9 +472,9 @@ print_V(const char *fname, FILE *fp, ber_tlv_tag_t tlv_tag, ber_tlv_len_t tlv_le
 	case ASN_BASIC_OBJECT_IDENTIFIER:
 	case ASN_BASIC_RELATIVE_OID:
 		if(tlv_len > 0 && tlv_len < 128*1024 /* VERY long OID! */) {
-			arcs = malloc(sizeof(*arcs) * (tlv_len + 1));
+			arcs = MALLOC(sizeof(*arcs) * (tlv_len + 1));
 			if(arcs) {
-				vbuf = malloc(tlv_len + 1);
+				vbuf = MALLOC(tlv_len + 1);
 				/* Not checking is intentional */
 			}
 		}
@@ -449,7 +503,7 @@ print_V(const char *fname, FILE *fp, ber_tlv_tag_t tlv_tag, ber_tlv_len_t tlv_le
 			(BER_TAG_CLASS(tlv_tag) != ASN_TAG_CLASS_UNIVERSAL
 				&& pretty_printing)
 		) && (tlv_len > 0 && tlv_len < 128 * 1024)) {
-			vbuf = malloc(tlv_len + 1);
+			vbuf = MALLOC(tlv_len + 1);
 			/* Not checking is intentional */
 		}
 		break;
@@ -467,8 +521,8 @@ print_V(const char *fname, FILE *fp, ber_tlv_tag_t tlv_tag, ber_tlv_len_t tlv_le
 		if(ch == -1) {
 			fprintf(stderr,
 			"%s: Unexpected end of file (V)\n", fname);
-			if(vbuf) free(vbuf);
-			if(arcs) free(arcs);
+			if(vbuf) FREEMEM(vbuf);
+			if(arcs) FREEMEM(arcs);
 			return -1;
 		}
 		switch(etype) {
@@ -537,7 +591,7 @@ print_V(const char *fname, FILE *fp, ber_tlv_tag_t tlv_tag, ber_tlv_len_t tlv_le
 					if(i) printf(".");
 					printf("%" PRIuASN, arcs[i]);
 				}
-				free(vbuf);
+				FREEMEM(vbuf);
 				vbuf = 0;
 			}
 		}
@@ -559,7 +613,7 @@ print_V(const char *fname, FILE *fp, ber_tlv_tag_t tlv_tag, ber_tlv_len_t tlv_le
 					if(i) printf(".");
 					printf("%" PRIuASN, arcs[i]);
 				}
-				free(vbuf);
+				FREEMEM(vbuf);
 				vbuf = 0;
 			}
 		}
@@ -602,10 +656,10 @@ print_V(const char *fname, FILE *fp, ber_tlv_tag_t tlv_tag, ber_tlv_len_t tlv_le
 			else
 				printf("%c", vbuf[i]);
 		}
-		free(vbuf);
+		FREEMEM(vbuf);
 	}
 
-	if(arcs) free(arcs);
+	if(arcs) FREEMEM(arcs);
 	return 0;
 }
 
@@ -621,7 +675,7 @@ decode_tlv_from_string(const char *datastring) {
 	int half;
 
 	dsize = strlen(datastring) + 1;
-	dp = data = calloc(1, dsize);
+	dp = data = CALLOC(1, dsize);
 	assert(data);
 
 	for(half = 0, p = datastring; *p; p++) {
