@@ -13,7 +13,7 @@ asn1c_activate_dependency(asn1c_fdeps_t *deps, asn1c_fdeps_t *cur, const char *d
 		return 0;
 	if(!cur) cur = deps;
 
-	if(cur->used_somewhere)
+	if(cur->usage > FDEP_NOTUSED)
 		return 1;	/* Already activated */
 
 	fname = data;
@@ -37,7 +37,7 @@ asn1c_activate_dependency(asn1c_fdeps_t *deps, asn1c_fdeps_t *cur, const char *d
 	}
 
 	if(cur->filename && strcmp(cur->filename, fname) == 0) {
-		cur->used_somewhere = 1;
+		cur->usage = FDEP_REFERRED;
 
 		/* Activate subdependencies */
 		for(i = 0; i < cur->el_count; i++) {
@@ -62,11 +62,11 @@ asn1c_activate_dependency(asn1c_fdeps_t *deps, asn1c_fdeps_t *cur, const char *d
 
 asn1c_fdeps_t *
 asn1c_read_file_dependencies(arg_t *arg, const char *datadir) {
+	char buf[4096];
 	asn1c_fdeps_t *deps;
 	asn1c_fdeps_t *cur;
-	char buf[4096];
 	FILE *f;
-	int hit_COMMON_FILES = 0;
+	enum fdep_usage special_section = FDEP_NOTUSED;
 
 	(void)arg;
 
@@ -91,17 +91,28 @@ asn1c_read_file_dependencies(arg_t *arg, const char *datadir) {
 		for(p = strtok(buf, " \t\r\n"); p;
 				p = strtok(NULL, " \t\r\n")) {
 			asn1c_fdeps_t *d;
+
 			/*
-			 * If hit "COMMON-FILES:", treat everything else
-			 * as a huge dependency.
+			 * Special "prefix" section.
 			 */
-			if(strcmp(p, "COMMON-FILES:") == 0) {
-				hit_COMMON_FILES = 1;
+			if(strchr(p, ':')) {
+				special_section = FDEP_IGNORE;
+				if(strcmp(p, "COMMON-FILES:") == 0) {
+					special_section = FDEP_COMMON_FILES;
+				} else if(strcmp(p, "CONVERTER:") == 0) {
+					special_section = FDEP_CONVERTER;
+				} else if((arg->flags & A1C_GEN_PER)
+					  && strcmp(p, "CODEC-PER:") == 0) {
+					special_section = FDEP_CODEC_PER;
+				}
 				break;
 			}
+
+			if(special_section == FDEP_IGNORE)
+				continue;
+
 			d = asn1c_new_dep(p);
-			assert(d);
-			d->used_somewhere = hit_COMMON_FILES;
+			d->usage = special_section;
 
 			if(asn1c_dep_add(cur, d) == 1)
 				cur = d;
@@ -163,8 +174,9 @@ asn1c_deps_makelist(asn1c_fdeps_t *deps) {
 
 	dlist = asn1c_new_dep(0);
 
-	if(deps->filename && deps->used_somewhere) {
+	if(deps->filename && deps->usage != FDEP_NOTUSED) {
 		d = asn1c_new_dep(deps->filename);
+		d->usage = deps->usage;
 		asn1c_dep_add(dlist, d);
 	}
 
