@@ -47,12 +47,31 @@ enum asn1c_flags {
 	 * Do not generate courtesy #includes for external dependencies.
 	 */
 	A1C_NO_INCLUDE_DEPS	= 0x0200,
+	/*
+	 * Compile members of CHOICE as indirect pointers.
+	 */
+	A1C_INDIRECT_CHOICE	= 0x0400,
+	/*
+	 * -fskeletons-copy
+	 * Copy support files rather than symlink them.
+	 */
+	A1C_SKELETONS_COPY	= 0x0800,
+	/*
+	 * -gen-PER
+	 * Generate PER support code
+	 */
+	A1C_GEN_PER		= 0x1000,
+	/*
+	 * -pdu=auto
+	 * Generate PDU table
+	 */
+	A1C_PDU_AUTO		= 0x2000
 };
 
 /*
  * Compile the ASN.1 specification.
  */
 int asn1_compile(asn1p_t *asn, const char *datadir, enum asn1c_flags,
-	int argc, char **argv);
+	int argc, int optc, char **argv);
 
 #endif	/* ASN1_COMPILER_H */
