@@ -1,7 +1,5 @@
 #include "asn1fix_internal.h"
 
-#define	AFT_MAGIC_ANY	1	/* _fetch_tag() flag */
-
 static int _asn1f_check_if_tag_must_be_explicit(arg_t *arg, asn1p_expr_t *v);
 static int _asn1f_compare_tags(arg_t *arg, asn1p_expr_t *a, asn1p_expr_t *b);
 static int _asn1f_fix_type_tag(arg_t *arg, asn1p_expr_t *expr);
@@ -154,7 +152,7 @@ asn1f_fix_constr_ext(arg_t *arg) {
 
 	if(arg->mod->module_flags & MSF_EXTENSIBILITY_IMPLIED
 	&& ext_count == 0) {
-		v = asn1p_expr_new(0);
+		v = asn1p_expr_new(0, arg->mod);
 		if(v) {
 			v->Identifier = strdup("...");
 			v->expr_type = A1TC_EXTENSIBLE;
@@ -252,7 +250,8 @@ asn1f_fix_constr_tag(arg_t *arg, int fix_top_level) {
 static int
 _asn1f_fix_type_tag(arg_t *arg, asn1p_expr_t *expr) {
 	int must_explicit = _asn1f_check_if_tag_must_be_explicit(arg, expr);
-	int module_impl_tags = (arg->mod->module_flags & MSF_IMPLICIT_TAGS);
+	int module_impl_tags = (arg->mod->module_flags
+				& (MSF_IMPLICIT_TAGS | MSF_AUTOMATIC_TAGS));
 	int r_value = 0;
 
 	if(expr->tag.tag_mode == TM_DEFAULT) {
@@ -352,6 +351,8 @@ asn1f_check_constr_tags_distinct(arg_t *arg) {
 		if(expr->expr_type != ASN_CONSTR_SEQUENCE || v->marker.flags) {
 			asn1p_expr_t *nv;
 			for(nv = v; (nv = TQ_NEXT(nv, next));) {
+				DEBUG("S/C comparing tags %s s. %s",
+					v->Identifier, nv->Identifier);
 				if(_asn1f_compare_tags(arg, v, nv))
 					r_value = -1;
 				if(expr->expr_type == ASN_CONSTR_SEQUENCE
@@ -403,12 +404,15 @@ _asn1f_compare_tags(arg_t *arg, asn1p_expr_t *a, asn1p_expr_t *b) {
 	int ra, rb;
 	int ret;
 
-	ra = asn1f_fetch_outmost_tag(arg->asn, arg->mod, a, &ta, AFT_MAGIC_ANY);
-	rb = asn1f_fetch_outmost_tag(arg->asn, arg->mod, b, &tb, AFT_MAGIC_ANY);
+	ra = asn1f_fetch_outmost_tag(arg->asn, arg->mod, a,
+			&ta, AFT_IMAGINARY_ANY);
+	rb = asn1f_fetch_outmost_tag(arg->asn, arg->mod, b,
+			&tb, AFT_IMAGINARY_ANY);
 
 	/*
 	 * If both tags are explicitly or implicitly given, use them.
 	 */
+	DEBUG("Fetching outmost tags: %d, %d", ra, rb);
 	if(ra == 0 && rb == 0) {
 		/*
 		 * Simple case: fetched both tags.
@@ -445,7 +449,7 @@ _asn1f_compare_tags(arg_t *arg, asn1p_expr_t *a, asn1p_expr_t *b) {
 					"improper use of "
 					"EXTENSIBILITY IMPLIED flag "
 					"of module %s",
-					arg->mod->Identifier);
+					arg->mod->ModuleName);
 			}
 			return -1;
 		} else {
@@ -467,7 +471,8 @@ _asn1f_compare_tags(arg_t *arg, asn1p_expr_t *a, asn1p_expr_t *b) {
 
 		DEBUG(" %s is a type reference", a->Identifier);
 
-		a = asn1f_lookup_symbol(arg, a->module, a->reference);
+		a = asn1f_lookup_symbol(arg,
+			a->module, a->rhs_pspecs, a->reference);
 		if(!a) return 0;	/* Already FATAL()'ed somewhere else */
 		WITH_MODULE(a->module, ret = _asn1f_compare_tags(arg, a, b));
 		return ret;
