@@ -11,11 +11,10 @@ typedef enum asn1p_expr_meta {
 	AMT_INVALID,
 	AMT_TYPE,		/* Type1 ::= INTEGER */
 	AMT_TYPEREF,		/* Type2 ::= Type1 */
-	AMT_PARAMTYPE,		/* Type3{Parameter} ::= SET { ... } */
 	AMT_VALUE,		/* value1 Type1 ::= 1 */
 	AMT_VALUESET,		/* ValueSet Type1 ::= { value1 } */
-	AMT_OBJECT,		/* FUNCTION ::= CLASS {...} */
-	AMT_OBJECTSET,		/* Functions FUNCTION ::= {...} */
+	AMT_OBJECT,		/* object CLASS ::= {...} */
+	AMT_OBJECTCLASS,	/* FUNCTION ::= CLASS {...} */
 	AMT_OBJECTFIELD,	/* ... */
 	AMT_EXPR_META_MAX
 } asn1p_expr_meta_e;
@@ -35,17 +34,26 @@ typedef enum asn1p_expr_type {
 	A1TC_OPAQUE,		/* Opaque data encoded as a bitvector */
 	A1TC_EXTENSIBLE,	/* An extension marker "..." */
 	A1TC_COMPONENTS_OF,	/* COMPONENTS OF clause */
-	A1TC_PARAMETRIZED,	/* A parametrized type declaration */
 	A1TC_VALUESET,		/* Value set definition */
 	A1TC_CLASSDEF,		/* Information Object Class */
-	A1TC_CLASSFIELD,	/* Information Object Class field */
 	A1TC_INSTANCE,		/* Instance of Object Class */
-	A1TC_TYPEID,		/* Type identifier */
+
+	/*
+	 * ASN.1 Class field types
+	 */
+#define	ASN_CLASSFIELD_MASK	0x10	/* Every class field type */
+	A1TC_CLASSFIELD_TFS	= ASN_CLASSFIELD_MASK,	/* TypeFieldSpec */
+	A1TC_CLASSFIELD_FTVFS,		/* FixedTypeValueFieldSpec */
+	A1TC_CLASSFIELD_VTVFS,		/* VariableTypeValueFieldSpec */
+	A1TC_CLASSFIELD_FTVSFS,		/* FixedTypeValueSetFieldSpec */
+	A1TC_CLASSFIELD_VTVSFS,		/* VariableTypeValueSetFieldSpec */
+	A1TC_CLASSFIELD_OFS,		/* ObjectFieldSpec */
+	A1TC_CLASSFIELD_OSFS,		/* ObjectSetFieldSpec */
 
 	/*
 	 * ASN.1 Constructed types
 	 */
-#define	ASN_CONSTR_MASK		0x10	/* Every constructed type */
+#define	ASN_CONSTR_MASK		0x20	/* Every constructed type */
 	ASN_CONSTR_SEQUENCE	= ASN_CONSTR_MASK,	/* SEQUENCE */
 	ASN_CONSTR_CHOICE,		/* CHOICE */
 	ASN_CONSTR_SET,			/* SET */
@@ -55,7 +63,7 @@ typedef enum asn1p_expr_type {
 	/*
 	 * ASN.1 Basic types
 	 */
-#define	ASN_BASIC_MASK		0x20	/* Every basic type */
+#define	ASN_BASIC_MASK		0x40	/* Every basic type */
 	ASN_TYPE_ANY		= ASN_BASIC_MASK,	/* ANY (deprecated) */
 	ASN_BASIC_BOOLEAN,
 	ASN_BASIC_NULL,
@@ -75,9 +83,9 @@ typedef enum asn1p_expr_type {
 	/*
 	 * ASN.1 String types
 	 */
-#define	ASN_STRING_KM_MASK	0x40	/* Known multiplier */
-#define	ASN_STRING_NKM_MASK	0x80	/* Not a known multiplier */
-#define	ASN_STRING_MASK		0xC0	/* Every restricted string type */
+#define	ASN_STRING_KM_MASK	0x100	/* Known multiplier */
+#define	ASN_STRING_NKM_MASK	0x200	/* Not a known multiplier */
+#define	ASN_STRING_MASK		0x300	/* Every restricted string type */
 	ASN_STRING_IA5String	= ASN_STRING_KM_MASK,
 	ASN_STRING_PrintableString,
 	ASN_STRING_VisibleString,
@@ -138,10 +146,27 @@ typedef struct asn1p_expr_s {
 	asn1p_constraint_t *combined_constraints;
 
 	/*
-	 * A list of parameters for parametrized type declaration
-	 * (AMT_PARAMTYPE).
+	 * Left hand side parameters for parametrized type declaration
+	 * Type{Param1, Param2} ::= SEQUENCE { a Param1, b Param2 }
 	 */
-	asn1p_paramlist_t *params;
+	asn1p_paramlist_t *lhs_params;
+	/*
+	 * Right hand type specialization.
+	 * Type2 ::= Type{Param1}
+	 */
+	struct asn1p_expr_s *rhs_pspecs;	/* ->members */
+	/*
+	 * If lhs_params is defined, this structure represents all possible
+	 * specializations of the parent expression.
+	 */
+	struct {
+		struct asn1p_pspec_s {
+			struct asn1p_expr_s *rhs_pspecs;
+			struct asn1p_expr_s *my_clone;
+		} *pspec;
+		int pspecs_count;	/* Number of specializations */
+	} specializations;
+	int spec_index;	/* -1, or 0-based specialization index in the parent */
 
 	/*
 	 * The actual value (DefinedValue or inlined value).
@@ -153,6 +178,13 @@ typedef struct asn1p_expr_s {
 	 */
 	asn1p_wsyntx_t *with_syntax;
 
+	/* Information Object Class matrix, specific for this class */
+	struct asn1p_ioc_matrix_s {
+		asn1p_ioc_row_t **row;
+		int rows;
+		int max_identifier_length;
+	} object_class_matrix;
+
 	/*
 	 * A tag.
 	 */
@@ -174,11 +206,12 @@ typedef struct asn1p_expr_s {
 
 	struct asn1p_expr_marker_s {
 		enum asn1p_expr_marker_e {
-			EM_NOMARK,
-			EM_INDIRECT	= 0x01,	/* 0001: Represent as pointer */
-			EM_OPTIONAL	= 0x03,	/* 0011: Optional member */
-			EM_DEFAULT	= 0x07,	/* 0111: default_value */
-			EM_UNRECURSE	= 0x08, /* 1000: Use safe naming */
+		  EM_NOMARK,
+		  EM_INDIRECT	= 0x01,	/* 00001 Represent as pointer */
+		  EM_OMITABLE	= 0x02,	/* 00010 May be absent in encoding */
+		  EM_OPTIONAL	= 0x07,	/* 00111 Optional member */
+		  EM_DEFAULT	= 0x0F,	/* 01111 default_value */
+		  EM_UNRECURSE	= 0x10,	/* 10000 Use safe naming */
 		} flags;
 		asn1p_value_t *default_value;	/* For EM_DEFAULT case */
 	} marker;
@@ -224,6 +257,7 @@ typedef struct asn1p_expr_s {
 	 */
 	int _anonymous_type;	/* This type is unnamed */
 	int _type_unique_index;	/* A per top-level-type unique index */
+	int _type_referenced;	/* This type is referenced from another place */
 
 	/*
 	 * Opaque data may be attached to this structure,
@@ -237,8 +271,11 @@ typedef struct asn1p_expr_s {
 /*
  * Constructor and destructor.
  */
-asn1p_expr_t *asn1p_expr_new(int _lineno);
+asn1p_expr_t *asn1p_expr_new(int _lineno, struct asn1p_module_s *);
 asn1p_expr_t *asn1p_expr_clone(asn1p_expr_t *, int skip_extensions);
+asn1p_expr_t *asn1p_expr_clone_with_resolver(asn1p_expr_t *,
+	asn1p_expr_t *(*resolver)(asn1p_expr_t *to_resolve, void *resolver_arg),
+	void *resolver_arg);
 void asn1p_expr_add(asn1p_expr_t *to, asn1p_expr_t *what);
 void asn1p_expr_free(asn1p_expr_t *expr);
 
