@@ -12,8 +12,8 @@ asn1p_module_new() {
 
 	mod = calloc(1, sizeof *mod);
 	if(mod) {
-		TQ_INIT(&(mod->imports));
 		TQ_INIT(&(mod->exports));
+		TQ_INIT(&(mod->imports));
 		TQ_INIT(&(mod->members));
 	}
 	return mod;
@@ -27,8 +27,8 @@ asn1p_module_free(asn1p_module_t *mod) {
 	if(mod) {
 		asn1p_expr_t *expr;
 
-		if(mod->Identifier)
-			free(mod->Identifier);
+		if(mod->ModuleName)
+			free(mod->ModuleName);
 
 		if(mod->module_oid)
 			asn1p_oid_free(mod->module_oid);
@@ -52,7 +52,7 @@ asn1p_new() {
 
 
 void
-asn1p_free(asn1p_t *asn) {
+asn1p_delete(asn1p_t *asn) {
 	if(asn) {
 		asn1p_module_t *mod;
 		while((mod = TQ_REMOVE(&(asn->modules), mod_next)))
