@@ -6,9 +6,14 @@
 #define	_REENTRANT			/* for Sun */
 #include <asn_internal.h>
 #include <GeneralizedTime.h>
-#include <time.h>
 #include <errno.h>
 
+#ifdef	__CYGWIN__
+#include "/usr/include/time.h"
+#else
+#include <time.h>
+#endif	/* __CYGWIN__ */
+
 #if	defined(WIN32)
 #pragma message( "PLEASE STOP AND READ!")
 #pragma message( "  localtime_r is implemented via localtime(), which may be not thread-safe.")
@@ -16,7 +21,7 @@
 #pragma message( "  ")
 #pragma message( "  You must fix the code by inserting appropriate locking")
 #pragma message( "  if you want to use asn_GT2time() or asn_UT2time().")
-#pragma message( "PLEASE STOP AND READ! ")
+#pragma message( "PLEASE STOP AND READ!")
 
 static struct tm *localtime_r(const time_t *tloc, struct tm *result) {
 	struct tm *tm;
@@ -38,7 +43,7 @@ static struct tm *gmtime_r(const time_t *tloc, struct tm *result) {
 
 #endif	/* WIN32 */
 
-#if	defined(sun)
+#if	defined(sun) || defined(_sun_) || defined(__solaris__)
 #define	_EMULATE_TIMEGM
 #endif
 
@@ -62,6 +67,15 @@ static struct tm *gmtime_r(const time_t *tloc, struct tm *result) {
 #define	GMTOFF(tm)	(-timezone)
 #endif	/* HAVE_TM_GMTOFF */
 
+#if	(defined(_EMULATE_TIMEGM) || !defined(HAVE_TM_GMTOFF))
+#warning "PLEASE STOP AND READ!"
+#warning "  timegm() is implemented via getenv(\"TZ\")/setenv(\"TZ\"), which may be not thread-safe."
+#warning "  "
+#warning "  You must fix the code by inserting appropriate locking"
+#warning "  if you want to use asn_GT2time() or asn_UT2time()."
+#warning "PLEASE STOP AND READ!"
+#endif	/* _EMULATE_TIMEGM */
+
 /*
  * Override our GMTOFF decision for other known platforms.
  */
@@ -84,25 +98,40 @@ static long GMTOFF(struct tm a){
 
 #endif	/* __CYGWIN__ */
 
+#define	ATZVARS do {							\
+	char tzoldbuf[64];						\
+	char *tzold
+#define	ATZSAVETZ do {							\
+	tzold = getenv("TZ");						\
+	if(tzold) {							\
+		size_t tzlen = strlen(tzold);				\
+		if(tzlen < sizeof(tzoldbuf))				\
+			tzold = memcpy(tzoldbuf, tzold, tzlen + 1);	\
+		else							\
+			tzold = strdup(tzold);	/* Ignore error */	\
+		setenv("TZ", "UTC", 1);					\
+	}								\
+	tzset();							\
+} while(0)
+#define	ATZOLDTZ do {							\
+	if (tzold) {							\
+		setenv("TZ", tzold, 1);					\
+		*tzoldbuf = 0;						\
+		if(tzold != tzoldbuf)					\
+			FREEMEM(tzold);					\
+	} else {							\
+		unsetenv("TZ");						\
+	}								\
+	tzset();							\
+} while(0); } while(0);
+
 #ifdef	_EMULATE_TIMEGM
 static time_t timegm(struct tm *tm) {
 	time_t tloc;
-	char *tz;
-	char *buf;
-
-	tz = getenv("TZ");
-	putenv("TZ=UTC");
-	tzset();
+	ATZVARS;
+	ATZSAVETZ;
 	tloc = mktime(tm);
-	if (tz) {
-		int bufsize = strlen(tz) + 4;
-		buf = alloca(bufsize);
-		snprintf(buf, bufsize, "TZ=%s", tz);
-	} else {
-		buf = "TZ=";
-	}
-	putenv(buf);
-	tzset();
+	ATZOLDTZ;
 	return tloc;
 }
 #endif	/* _EMULATE_TIMEGM */
@@ -128,6 +157,7 @@ asn_TYPE_descriptor_t asn_DEF_GeneralizedTime = {
 	GeneralizedTime_encode_der,
 	OCTET_STRING_decode_xer_utf8,
 	GeneralizedTime_encode_xer,
+	0, 0,
 	0, /* Use generic outmost tag fetcher */
 	asn_DEF_GeneralizedTime_tags,
 	sizeof(asn_DEF_GeneralizedTime_tags)
@@ -135,6 +165,7 @@ asn_TYPE_descriptor_t asn_DEF_GeneralizedTime = {
 	asn_DEF_GeneralizedTime_tags,
 	sizeof(asn_DEF_GeneralizedTime_tags)
 	  / sizeof(asn_DEF_GeneralizedTime_tags[0]),
+	0,	/* No PER visible constraints */
 	0, 0,	/* No members */
 	0	/* No specifics */
 };
@@ -146,14 +177,14 @@ asn_TYPE_descriptor_t asn_DEF_GeneralizedTime = {
  */
 int
 GeneralizedTime_constraint(asn_TYPE_descriptor_t *td, const void *sptr,
-		asn_app_consume_bytes_f *app_errlog, void *app_key) {
+		asn_app_constraint_failed_f *ctfailcb, void *app_key) {
 	const GeneralizedTime_t *st = (const GeneralizedTime_t *)sptr;
 	time_t tloc;
 
 	errno = EPERM;			/* Just an unlikely error code */
 	tloc = asn_GT2time(st, 0, 0);
 	if(tloc == -1 && errno != EPERM) {
-		_ASN_ERRLOG(app_errlog, app_key,
+		_ASN_CTFAIL(app_key, td,
 			"%s: Invalid time format: %s (%s:%d)",
 			td->name, strerror(errno), __FILE__, __LINE__);
 		return -1;
@@ -216,8 +247,7 @@ GeneralizedTime_encode_xer(asn_TYPE_descriptor_t *td, void *sptr,
 	
 		rv = OCTET_STRING_encode_xer_utf8(td, sptr, ilevel, flags,
 			cb, app_key);
-		asn_DEF_GeneralizedTime.free_struct(&asn_DEF_GeneralizedTime,
-			gt, 0);
+		ASN_STRUCT_FREE(asn_DEF_GeneralizedTime, gt);
 		return rv;
 	} else {
 		return OCTET_STRING_encode_xer_utf8(td, sptr, ilevel, flags,
@@ -324,7 +354,7 @@ asn_GT2time_frac(const GeneralizedTime_t *st, int *frac_value, int *frac_digits,
 #undef	B2T
 #define	B2F(var)	do {					\
 		unsigned ch = *buf;				\
-		if(ch < 0x30 && ch > 0x39) {			\
+		if(ch < 0x30 || ch > 0x39) {			\
 			errno = EINVAL;				\
 			return -1;				\
 		} else {					\
@@ -566,8 +596,8 @@ asn_time2GT_frac(GeneralizedTime_t *opt_gt, const struct tm *tm, int frac_value,
 		tm = &tm_s;
 #ifdef	HAVE_TM_GMTOFF
 		assert(!GMTOFF(tm_s));	/* Will fix itself */
-#else
-		gmtoff = 0;		/* Intervention required */
+#else	/* !HAVE_TM_GMTOFF */
+		gmtoff = 0;
 #endif
 	}
 
@@ -641,7 +671,7 @@ asn_time2GT_frac(GeneralizedTime_t *opt_gt, const struct tm *tm, int frac_value,
 			FREEMEM(opt_gt->buf);
 	} else {
 		opt_gt = (GeneralizedTime_t *)CALLOC(1, sizeof *opt_gt);
-		if(!opt_gt) { free(buf); return 0; }
+		if(!opt_gt) { FREEMEM(buf); return 0; }
 	}
 
 	opt_gt->buf = (unsigned char *)buf;
