@@ -3,7 +3,7 @@
  * Redistribution and modifications are permitted subject to BSD license.
  */
 /*
- * Please read the NativeReal.h for the explanation wrt. differences between
+ * Read the NativeReal.h for the explanation wrt. differences between
  * REAL and NativeReal.
  * Basically, both are decoders and encoders of ASN.1 REAL type, but this
  * implementation deals with the standard (machine-specific) representation
@@ -29,11 +29,13 @@ asn_TYPE_descriptor_t asn_DEF_NativeReal = {
 	NativeReal_encode_der,
 	NativeReal_decode_xer,
 	NativeReal_encode_xer,
+	0, 0,
 	0, /* Use generic outmost tag fetcher */
 	asn_DEF_NativeReal_tags,
 	sizeof(asn_DEF_NativeReal_tags) / sizeof(asn_DEF_NativeReal_tags[0]),
 	asn_DEF_NativeReal_tags,	/* Same as above */
 	sizeof(asn_DEF_NativeReal_tags) / sizeof(asn_DEF_NativeReal_tags[0]),
+	0,	/* No PER visible constraints */
 	0, 0,	/* No members */
 	0	/* No specifics */
 };
@@ -150,7 +152,7 @@ NativeReal_encode_der(asn_TYPE_descriptor_t *td, void *ptr,
 	}
 
 	/* Free possibly allocated members of the temporary structure */
-	asn_DEF_REAL.free_struct(&asn_DEF_REAL, &tmp, 1);
+	ASN_STRUCT_FREE_CONTENTS_ONLY(asn_DEF_REAL, &tmp);
 
 	return erval;
 }
@@ -166,6 +168,7 @@ NativeReal_decode_xer(asn_codec_ctx_t *opt_codec_ctx,
 		const void *buf_ptr, size_t size) {
 	asn_dec_rval_t rval;
 	REAL_t *st = 0;
+	REAL_t **stp = &st;
 	double *Dbl = (double *)*sptr;
 
 	if(!Dbl) {
@@ -178,7 +181,7 @@ NativeReal_decode_xer(asn_codec_ctx_t *opt_codec_ctx,
 		}
 	}
 
-	rval = REAL_decode_xer(opt_codec_ctx, td, (void **)&st, opt_mname,
+	rval = REAL_decode_xer(opt_codec_ctx, td, (void **)stp, opt_mname,
 		buf_ptr, size);
 	if(rval.code == RC_OK) {
 		if(asn_REAL2double(st, Dbl)) {
@@ -188,7 +191,7 @@ NativeReal_decode_xer(asn_codec_ctx_t *opt_codec_ctx,
 	} else {
 		rval.consumed = 0;
 	}
-	asn_DEF_REAL.free_struct(&asn_DEF_REAL, st, 0);
+	ASN_STRUCT_FREE(asn_DEF_REAL, st);
 	return rval;
 }
 
@@ -206,7 +209,7 @@ NativeReal_encode_xer(asn_TYPE_descriptor_t *td, void *sptr,
 	er.encoded = REAL__dump(*Dbl, flags & XER_F_CANONICAL, cb, app_key);
 	if(er.encoded < 0) _ASN_ENCODE_FAILED;
 
-	return er;
+	_ASN_ENCODED_OK(er);
 }
 
 /*
