@@ -42,11 +42,13 @@ asn_TYPE_descriptor_t asn_DEF_REAL = {
 	der_encode_primitive,
 	REAL_decode_xer,
 	REAL_encode_xer,
+	0, 0,
 	0, /* Use generic outmost tag fetcher */
 	asn_DEF_REAL_tags,
 	sizeof(asn_DEF_REAL_tags) / sizeof(asn_DEF_REAL_tags[0]),
 	asn_DEF_REAL_tags,	/* Same as above */
 	sizeof(asn_DEF_REAL_tags) / sizeof(asn_DEF_REAL_tags[0]),
+	0,	/* No PER visible constraints */
 	0, 0,	/* No members */
 	0	/* No specifics */
 };
@@ -117,7 +119,7 @@ REAL__dump(double d, int canonical, asn_app_consume_bytes_f *cb, void *app_key)
 			buflen = ret;
 			break;
 		}
-		if(buf != local_buf) free(buf);
+		if(buf != local_buf) FREEMEM(buf);
 		buf = (char *)MALLOC(buflen);
 		if(!buf) return -1;
 	} while(1);
@@ -217,7 +219,7 @@ REAL__dump(double d, int canonical, asn_app_consume_bytes_f *cb, void *app_key)
 	}
 
 	ret = cb(buf, buflen, app_key);
-	if(buf != local_buf) free(buf);
+	if(buf != local_buf) FREEMEM(buf);
 	return (ret < 0) ? -1 : buflen;
 }
 
@@ -257,7 +259,7 @@ REAL_encode_xer(asn_TYPE_descriptor_t *td, void *sptr,
 	er.encoded = REAL__dump(d, flags & XER_F_CANONICAL, cb, app_key);
 	if(er.encoded < 0) _ASN_ENCODE_FAILED;
 
-	return er;
+	_ASN_ENCODED_OK(er);
 }
 
 
@@ -320,7 +322,7 @@ REAL__xer_body_decode(asn_TYPE_descriptor_t *td, void *sptr, const void *chunk_b
 	b[chunk_size] = 0;	/* nul-terminate */
 
 	value = strtod(b, &endptr);
-	free(b);
+	FREEMEM(b);
 	if(endptr == b) return XPBD_BROKEN_ENCODING;
 
 	if(asn_double2REAL(st, value))
