@@ -59,13 +59,15 @@ encode(time_t tloc, const char *expect, int force_gmt) {
 	}
 }
 
+#define	RECODE(foo, bar)	recode(__LINE__, foo, bar)
 
 static void
-recode(char *time_str, const char *expect) {
+recode(int lineno, char *time_str, const char *expect) {
 	int frac_value, frac_digits;
 	GeneralizedTime_t gt;
 	struct tm tm;
 	time_t tloc;
+	char *tz;
 
 	gt.buf = (uint8_t *)time_str;
 	gt.size = strlen(time_str);
@@ -77,8 +79,11 @@ recode(char *time_str, const char *expect) {
 	asn_time2GT_frac(&gt, &tm, frac_value, frac_digits, 1);
 	assert(gt.buf);
 
-	printf("[%s] => [%s] == [%s] (%d, %d)\n",
-		time_str, gt.buf, expect, frac_value, frac_digits);
+	tz = getenv("TZ");
+	printf("%d: [%s] (%ld) => [%s] == [%s] (%d, %d) (TZ=%s)\n",
+		lineno, time_str, (long)tloc, gt.buf,
+		expect, frac_value, frac_digits,
+		tz ? tz : "");
 
 	assert(strcmp((char *)gt.buf, expect) == 0);
 	FREEMEM(gt.buf);
@@ -182,9 +187,12 @@ check_fractions() {
 
 int
 main(int ac, char **av) {
+	char *tz = getenv("TZ");
 
 	(void)av;
 
+	printf("TZ = [%s]\n", tz ? tz : "");
+
 	check_fractions();
 
 	recognize("200401250", -1, 0);
@@ -222,32 +230,36 @@ main(int ac, char **av) {
 		recognize("20040125093000,1234", 1075051800, 0);
 
 		encode(1075023007, "20040125013007-0800", 0);
-		recode("20050702123312", "20050702193312Z");
+		RECODE("20050702123312", "20050702193312Z");
 	}
 
-	recode("20050702123312Z", "20050702123312Z");
-	recode("20050702123312+01", "20050702113312Z");
-	recode("20050702123312,0+01", "20050702113312Z");
-	recode("20050702123312,1+01", "20050702113312.1Z");
-	recode("20050702123312.01+01", "20050702113312.01Z");
-	recode("20050702123312.00+01", "20050702113312Z");
-	recode("20050702123312.30+01", "20050702113312.3Z");
-	recode("20050702123312,30000+01", "20050702113312.3Z");
-	recode("20050702123312,300000000+01", "20050702113312.3Z");
-	recode("20050702123312.123456+01", "20050702113312.123456Z");
-	recode("20050702123312.1234567+01", "20050702113312.123456Z");
-	recode("20050702123312.12345678+01", "20050702113312.123456Z");
-	recode("20050702123312.123456789+01", "20050702113312.123456Z");
-	recode("20050702123312.2000000000+01", "20050702113312.2Z");
-	recode("20050702123312.3000000000+01", "20050702113312.3Z");
-	recode("20050702123312.4000000000+01", "20050702113312.4Z");
-	recode("20050702123312.5000000000+01", "20050702113312.5Z");
-	recode("20050702123312.5000000001+01", "20050702113312.5Z");
-	recode("20050702123312.5000010001+01", "20050702113312.500001Z");
-	recode("20050702123312.5000001001+01", "20050702113312.5Z");
-	recode("20050702123312.000001+01", "20050702113312.000001Z");
-	recode("20050702123312.0000001Z", "20050702123312Z");
-	recode("20050702123312.0080010+1056", "20050702013712.008001Z");
+#if defined(sun) || defined(__sun) || defined(_sun_) || defined(__solaris__)
+	printf("Solaris does not have a decent timegm() function.\n");
+#else	/* !solaris */
+	RECODE("20050702123312Z", "20050702123312Z");
+	RECODE("20050702123312+01", "20050702113312Z");
+	RECODE("20050702123312,0+01", "20050702113312Z");
+	RECODE("20050702123312,1+01", "20050702113312.1Z");
+	RECODE("20050702123312.01+01", "20050702113312.01Z");
+	RECODE("20050702123312.00+01", "20050702113312Z");
+	RECODE("20050702123312.30+01", "20050702113312.3Z");
+	RECODE("20050702123312,30000+01", "20050702113312.3Z");
+	RECODE("20050702123312,300000000+01", "20050702113312.3Z");
+	RECODE("20050702123312.123456+01", "20050702113312.123456Z");
+	RECODE("20050702123312.1234567+01", "20050702113312.123456Z");
+	RECODE("20050702123312.12345678+01", "20050702113312.123456Z");
+	RECODE("20050702123312.123456789+01", "20050702113312.123456Z");
+	RECODE("20050702123312.2000000000+01", "20050702113312.2Z");
+	RECODE("20050702123312.3000000000+01", "20050702113312.3Z");
+	RECODE("20050702123312.4000000000+01", "20050702113312.4Z");
+	RECODE("20050702123312.5000000000+01", "20050702113312.5Z");
+	RECODE("20050702123312.5000000001+01", "20050702113312.5Z");
+	RECODE("20050702123312.5000010001+01", "20050702113312.500001Z");
+	RECODE("20050702123312.5000001001+01", "20050702113312.5Z");
+	RECODE("20050702123312.000001+01", "20050702113312.000001Z");
+	RECODE("20050702123312.0000001Z", "20050702123312Z");
+	RECODE("20050702123312.0080010+1056", "20050702013712.008001Z");
+#endif
 
 	return 0;
 }
@@ -267,6 +279,7 @@ OCTET_STRING_encode_der(asn_TYPE_descriptor_t *td, void *ptr, int tag_mode, ber_
 	(void)cb;
 	(void)app_key;
 
+	memset(&erval, 0, sizeof(erval));
 	return erval;
 }
 
@@ -281,5 +294,6 @@ OCTET_STRING_encode_xer_utf8(asn_TYPE_descriptor_t *td, void *ptr, int ilevel, e
 	(void)cb;
 	(void)app_key;
 
+	memset(&erval, 0, sizeof(erval));
 	return erval;
 }
